#include <math.h>
#include <imsls.h>
#include <stdio.h>
#include <stdlib.h>

void l_print_LSD(int n1, int* equalMeans, float *means);

int main()
{
   char **anova_row_labels;
   char *col_labels[] = {" ", "\nID", "\nDF", "\nSSQ  ", 
      "Mean  \nsquares", "\nF-Test", "\np-Value"};
   int i, l, page_width=100;
   int n            = 16; /* Total number of observations */
   int n_treatments = 4;
   int n_locations  = 1;
   int df, *equal_means;
   float grand_mean, cv;
   float *aov, *treatment_means, *std_err;
   int col[]={1, 2, 3, 4, 1, 2, 3, 4, 1, 2, 3, 4, 1, 2, 3, 4};
   int row[]={3, 2, 4, 1, 1, 4, 2, 3, 2, 3, 1, 4, 4, 1, 3, 2};
   int treatment[]={1, 1, 1, 1, 2, 2, 2, 2, 3, 3, 3, 3, 4, 4, 4, 4};
   float alpha = 0.05;
   float y[]={ 
      1.167,  1.185,  1.655, 1.345, 1.64, 1.29, 1.665, 1.29,
      1.475, 0.71, 1.425, 0.66, 1.565, 1.29, 1.4, 1.18
   };
 
   printf("\n\n*** Experimental Design ***");
   printf("\n===============================");
   printf("\n| COL  |  1  |  2  |  3  |  4  |");
   printf("\n===============================");
   printf("\n|ROW 1 |  2  |  4  |  3  |  1  |");
   printf("\n===============================");
   printf("\n|ROW 2 |  3  |  1  |  2  |  4  |");
   printf("\n===============================");
   printf("\n|ROW 3 |  1  |  3  |  4  |  2  |");
   printf("\n===============================");
   printf("\n|ROW 4 |  4  |  2  |  1  |  3  |");
   printf("\n===============================");
 
   aov = imsls_f_latin_square(n, n_locations, n_treatments,  
      row, col, treatment, y, 
      IMSLS_GRAND_MEAN, &grand_mean, 
      IMSLS_CV, &cv,
      IMSLS_TREATMENT_MEANS, &treatment_means, 
      IMSLS_STD_ERRORS, &std_err,
      IMSLS_ANOVA_ROW_LABELS, &anova_row_labels,
      0);

   /* Print ANOVA table. */
   imsls_page(IMSLS_SET_PAGE_WIDTH, &page_width);
   imsls_f_write_matrix("\n   *** ANALYSIS OF VARIANCE TABLE ***", 
      7, 6, aov, 
      IMSLS_WRITE_FORMAT, "%3.0f%3.0f%8.3f%8.3f%8.3f%8.3f",
      IMSLS_ROW_LABELS, anova_row_labels,
      IMSLS_COL_LABELS, col_labels,
      0);

   printf("\n\nGrand Mean:               %7.3f", grand_mean);
   printf("\n\nCoefficient of Variation: %7.3f\n\n", cv);
   l = 0;
   printf("Treatment Means: \n");
   for (i=0; i < n_treatments; i++){
      printf("treatment[%2d]              %7.4f \n", i+1, 
         treatment_means[l++]);
   }
   df = (int)std_err[1];
   printf("\n\n");
   printf("Standard Error for Comparing Two Treatment Means: %f \n",
      std_err[0]);
   printf("(df=%d)\n", df);
   equal_means = imsls_f_multiple_comparisons(n_treatments, 
      treatment_means, df, std_err[0]/sqrt(2.0), 
      IMSLS_LSD,
      IMSLS_ALPHA, alpha,
      0);

   l_print_LSD(n_treatments, equal_means, treatment_means);
}

/*
* Function to display means comparison.
*/
void l_print_LSD(int n, int *equalMeans, float *means){
    float x=0.0;
    int i, j, k;
    int iSwitch;
    int *idx;

    idx = (int *) malloc(n * sizeof (int));

    for (k=0; k < n; k++) {
        idx[k]   =k+1;
    }

    /* Sort means in ascending order*/

    iSwitch=1;
    while (iSwitch != 0){
        iSwitch = 0;
        for (i = 0; i < n-1; i++){
            if (means[i] > means[i+1]){
                iSwitch = 1;
                x = means[i];
                means[i] = means[i+1];
                means[i+1] = x;
                j = idx[i];
                idx[i] = idx[i+1];
                idx[i+1] = j;
            }
        }
    }
    printf("[group] \t  Mean \t\tLSD Grouping \n");
    for (i=0; i < n; i++){
        printf("  [%d] \t\t%f", idx[i], means[i]);                
        for (j=1; j < i+1; j++){
            if(equalMeans[j-1] >= i+2-j){
                printf("\t  *");
            }else{ 
                if(equalMeans[j-1]>0) printf("\t");
            }
        }
        if (i < n-1 && equalMeans[i]>0) printf("\t  *");
        printf("\n");
    }
    imsls_free(idx);
    idx = NULL;
    return;
}
